@@ -41,6 +41,7 @@
  * It was modified to fit into unbound. The state table process is the same.
  */
 #include "config.h"
+#include "ldns/ldns.h"
 #include "validator/autotrust.h"
 #include "validator/val_anchor.h"
 #include "validator/val_utils.h"
@@ -599,8 +600,7 @@ static struct trust_anchor*
 parse_id(struct val_anchors* anchors, char* line)
 {
 	struct trust_anchor *tp;
-	size_t len;
-	int labs, r;
+	int r;
 	ldns_rdf* rdf;
 	uint16_t dclass;
 	/* read the owner name */
@@ -611,8 +611,6 @@ parse_id(struct val_anchors* anchors, char* line)
 	rdf = ldns_dname_new_frm_str(line);
 	if(!rdf)
 		return NULL;
-	labs = dname_count_size_labels(ldns_rdf_data(rdf), &len);
-	log_assert(len == ldns_rdf_size(rdf));
 
 	/* read the class */
 	dclass = parse_int(next+1, &r);
@@ -940,8 +938,8 @@ void autr_write_file(struct module_env* env, struct trust_anchor* tp)
 			continue;
 		if(ta->s == AUTR_STATE_REMOVED)
 			continue;
-		/* only store SEP keys */
-		if(!rr_is_dnskey_sep(ta->rr))
+		/* only store keys */
+		if(ldns_rr_get_type(ta->rr) != LDNS_RR_TYPE_DNSKEY)
 			continue;
 		str = ldns_rr2str(ta->rr);
 		if(!str || !str[0]) {
@@ -1577,6 +1575,37 @@ anchor_state_update(struct module_env* env, struct autr_ta* anchor, int* c)
 	}
 }
 
+/** if ZSK init then trust KSKs */
+static int
+init_zsk_to_ksk(struct module_env* env, struct trust_anchor* tp, int* changed)
+{
+	/* search for VALID ZSKs */
+	struct autr_ta* anchor;
+	int validzsk = 0;
+	int validksk = 0;
+	for(anchor = tp->autr->keys; anchor; anchor = anchor->next) {
+		/* last_change test makes sure it was manually configured */
+                if (ldns_rr_get_type(anchor->rr) == LDNS_RR_TYPE_DNSKEY &&
+			anchor->last_change == 0 && 
+			!rr_is_dnskey_sep(anchor->rr) &&
+			anchor->s == AUTR_STATE_VALID)
+                        validzsk++;
+	}
+	if(validzsk == 0)
+		return 0;
+	for(anchor = tp->autr->keys; anchor; anchor = anchor->next) {
+                if (rr_is_dnskey_sep(anchor->rr) && 
+			anchor->s == AUTR_STATE_ADDPEND) {
+			verbose_key(anchor, VERB_ALGO, "trust KSK from "
+				"ZSK(config)");
+			set_trustanchor_state(env, anchor, changed, 
+				AUTR_STATE_VALID);
+			validksk++;
+		}
+	}
+	return validksk;
+}
+
 /** Remove missing trustanchors so the list does not grow forever */
 static void
 remove_missing_trustanchors(struct module_env* env, struct trust_anchor* tp,
@@ -1585,8 +1614,6 @@ remove_missing_trustanchors(struct module_env* env, struct trust_anchor* tp,
 	struct autr_ta* anchor;
 	int exceeded;
 	int valid = 0;
-	if(env->cfg->keep_missing == 0)
-		return; /* keep forever */
 	/* see if we have anchors that are valid */
 	for(anchor = tp->autr->keys; anchor; anchor = anchor->next) {
 		/* Only do KSKs */
@@ -1595,16 +1622,38 @@ remove_missing_trustanchors(struct module_env* env, struct trust_anchor* tp,
                 if (anchor->s == AUTR_STATE_VALID)
                         valid++;
 	}
-	if(valid == 0)
-		return;
+	/* if there are no SEP Valid anchors, see if we started out with
+	 * a ZSK (last-change=0) anchor, which is VALID and there are KSKs
+	 * now that can be made valid.  Do this immediately because there
+	 * is no guarantee that the ZSKs get announced long enough.  Usually
+	 * this is immediately after init with a ZSK trusted, unless the domain
+	 * was not advertising any KSKs at all.  In which case we perfectly
+	 * track the zero number of KSKs. */
+	if(valid == 0) {
+		valid = init_zsk_to_ksk(env, tp, changed);
+		if(valid == 0)
+			return;
+	}
 	
 	for(anchor = tp->autr->keys; anchor; anchor = anchor->next) {
-		/* Only do KSKs */
-                if (!rr_is_dnskey_sep(anchor->rr))
+		/* ignore ZSKs if newly added */
+		if(anchor->s == AUTR_STATE_START)
+			continue;
+		/* remove ZSKs if a KSK is present */
+                if (!rr_is_dnskey_sep(anchor->rr)) {
+			if(valid > 0) {
+				verbose_key(anchor, VERB_ALGO, "remove ZSK "
+					"[%d key(s) VALID]", valid);
+				set_trustanchor_state(env, anchor, changed, 
+					AUTR_STATE_REMOVED);
+			}
                         continue;
+		}
                 /* Only do MISSING keys */
                 if (anchor->s != AUTR_STATE_MISSING)
                         continue;
+		if(env->cfg->keep_missing == 0)
+			continue; /* keep forever */
 
 		exceeded = check_holddown(env, anchor, env->cfg->keep_missing);
 		/* If keep_missing has exceeded and we still have more than 
@@ -1656,7 +1705,7 @@ autr_cleanup_keys(struct trust_anchor* tp)
 	while(p) {
 		/* do we want to remove this key? */
 		if(p->s == AUTR_STATE_START || p->s == AUTR_STATE_REMOVED ||
-			!rr_is_dnskey_sep(p->rr)) {
+			ldns_rr_get_type(p->rr) != LDNS_RR_TYPE_DNSKEY) {
 			struct autr_ta* np = p->next;
 			/* remove */
 			ldns_rr_free(p->rr);
@@ -1684,7 +1733,7 @@ calc_next_probe(struct module_env* env, uint32_t wait)
 		wait = 3600;
 	rnd = wait/10;
 	rest = wait-rnd;
-	rnd = (uint32_t)ub_random(env->rnd) % rnd;
+	rnd = (uint32_t)ub_random_max(env->rnd, (long int)rnd);
 	return (time_t)(*env->now + rest + rnd);
 }
 
