@@ -41,6 +41,7 @@
  * bridging between RR wireformat data and crypto calls.
  */
 #include "config.h"
+#include "ldns/ldns.h"
 #include "validator/val_sigcrypt.h"
 #include "validator/validator.h"
 #include "util/data/msgreply.h"
@@ -55,6 +56,22 @@
 #error "Need SSL library to do digital signature cryptography"
 #endif
 
+#ifdef HAVE_OPENSSL_ERR_H
+#include <openssl/err.h>
+#endif
+
+#ifdef HAVE_OPENSSL_RAND_H
+#include <openssl/rand.h>
+#endif
+
+#ifdef HAVE_OPENSSL_CONF_H
+#include <openssl/conf.h>
+#endif
+
+#ifdef HAVE_OPENSSL_ENGINE_H
+#include <openssl/engine.h>
+#endif
+
 /** return number of rrs in an rrset */
 static size_t
 rrset_get_count(struct ub_packed_rrset_key* rrset)
@@ -257,7 +274,7 @@ ds_digest_size_algo(struct ub_packed_rrset_key* k, size_t idx)
 			return SHA256_DIGEST_LENGTH;
 #endif
 #ifdef USE_GOST
-		case LDNS_HASH_GOST94:
+		case LDNS_HASH_GOST:
 			if(EVP_get_digestbyname("md_gost94"))
 				return 32;
 			else	return 0;
@@ -268,7 +285,7 @@ ds_digest_size_algo(struct ub_packed_rrset_key* k, size_t idx)
 }
 
 #ifdef USE_GOST
-/** Perform GOST94 hash */
+/** Perform GOST hash */
 static int
 do_gost94(unsigned char* data, size_t len, unsigned char* dest)
 {
@@ -325,7 +342,7 @@ ds_create_dnskey_digest(struct module_env* env,
 			return 1;
 #endif
 #ifdef USE_GOST
-		case LDNS_HASH_GOST94:
+		case LDNS_HASH_GOST:
 			if(do_gost94((unsigned char*)ldns_buffer_begin(b), 
 				ldns_buffer_limit(b), (unsigned char*)digest))
 				return 1;
@@ -402,7 +419,7 @@ dnskey_algo_id_is_supported(int id)
 #endif
 		return 1;
 #ifdef USE_GOST
-	case LDNS_GOST:
+	case LDNS_ECC_GOST:
 		/* we support GOST if it can be loaded */
 		return ldns_key_EVP_load_gost_id();
 #endif
@@ -1236,15 +1253,12 @@ setup_dsa_sig(unsigned char** sig, unsigned int* len)
 	unsigned char* orig = *sig;
 	unsigned int origlen = *len;
 	int newlen;
-
-	uint8_t t;
 	BIGNUM *R, *S;
 	DSA_SIG *dsasig;
 
 	/* extract the R and S field from the sig buffer */
 	if(origlen < 1 + 2*SHA_DIGEST_LENGTH)
 		return 0;
-	t = orig[0];
 	R = BN_new();
 	if(!R) return 0;
 	(void) BN_bin2bn(orig + 1, SHA_DIGEST_LENGTH, R);
@@ -1366,7 +1380,7 @@ setup_key_digest(int algo, EVP_PKEY** evp_key, const EVP_MD** digest_type,
 
 			break;
 #ifdef USE_GOST
-		case LDNS_GOST:
+		case LDNS_ECC_GOST:
 			*evp_key = ldns_gost2pkey_raw(key, keylen);
 			if(!*evp_key) {
 				verbose(VERB_QUERY, "verify: "
